@@ -3,17 +3,71 @@ import torch
 import torch.nn as nn
 from ray.rllib.utils import override
 
+from raylab.options import configure
+from raylab.options import option
 from raylab.policy import TorchPolicy
 from raylab.policy.action_dist import WrapDeterministicPolicy
 from raylab.policy.losses import ActionDPG
 from raylab.policy.losses import DeterministicPolicyGradient
 from raylab.policy.losses import FittedQLearning
 from raylab.policy.modules.critic import HardValue
+from raylab.policy.off_policy import off_policy_options
+from raylab.policy.off_policy import OffPolicyMixin
 from raylab.torch.nn.utils import update_polyak
 from raylab.torch.optim import build_optimizer
+from raylab.utils.annotations import TensorDict
 
 
-class SOPTorchPolicy(TorchPolicy):
+@configure
+@off_policy_options
+@option(
+    "dpg_loss",
+    "default",
+    help="""
+    Type of Deterministic Policy Gradient to use.
+
+    'default' backpropagates Q-value gradients through the critic network.
+
+    'acme' uses Acme's implementation which recovers DPG via a MSE loss between
+    the actor's action and the action + Q-value gradient. Allows monitoring the
+    magnitude of the action-value gradient.""",
+)
+@option(
+    "dqda_clipping",
+    None,
+    help="""
+    Optional value by which to clip the action gradients. Only used with
+    dpg_loss='acme'.""",
+)
+@option(
+    "clip_dqda_norm",
+    False,
+    help="""
+    Whether to clip action grads by norm or value. Only used with
+    dpg_loss='acme'.""",
+)
+@option("torch_optimizer/actor", {"type": "Adam", "lr": 1e-3})
+@option("torch_optimizer/critics", {"type": "Adam", "lr": 1e-3})
+@option(
+    "polyak",
+    0.995,
+    help="Interpolation factor in polyak averaging for target networks.",
+)
+@option(
+    "policy_delay",
+    1,
+    help="Update policy every this number of calls to `learn_on_batch`",
+)
+@option("module/type", "DDPG")
+@option("module/actor/separate_behavior", True)
+@option("exploration_config/type", "raylab.utils.exploration.ParameterNoise")
+@option(
+    "exploration_config/param_noise_spec",
+    {"initial_stddev": 0.1, "desired_action_stddev": 0.2, "adaptation_coeff": 1.01},
+    help="Options for parameter noise exploration",
+)
+@option("exploration_config/pure_exploration_steps", 1000)
+class SOPTorchPolicy(OffPolicyMixin, TorchPolicy):
     """Streamlined Off-Policy policy in PyTorch to use with RLlib."""
 
     # pylint:disable=abstract-method
@@ -28,13 +82,7 @@ class SOPTorchPolicy(TorchPolicy):
         self.loss_critic.gamma = self.config["gamma"]
         self._grad_step = 0
 
-    @property
-    @override(TorchPolicy)
-    def options(self):
-        # pylint:disable=cyclic-import
-        from raylab.agents.sop import SOPTrainer
-
-        return SOPTrainer.options
+        self.build_replay_buffer()
 
     def _make_actor_loss(self):
         if self.config["dpg_loss"] == "default":
@@ -67,19 +115,17 @@ class SOPTorchPolicy(TorchPolicy):
         optimizers.update(mapping)
         return optimizers
 
-    @override(TorchPolicy)
-    def learn_on_batch(self, samples):
-        batch_tensors = self.lazy_tensor_dict(samples)
-
+    @override(OffPolicyMixin)
+    def improve_policy(self, batch: TensorDict):
         info = {}
         self._grad_step += 1
-        info.update(self._update_critic(batch_tensors))
-        if self._grad_step % self.config["policy_delay"] == 0:
-            info.update(self._update_policy(batch_tensors))
 
-        update_polyak(
-            self.module.critics, self.module.target_critics, self.config["polyak"]
-        )
+        info.update(self._update_critic(batch))
+        if self._grad_step % self.config["policy_delay"] == 0:
+            info.update(self._update_policy(batch))
+
+        critics, target_critics = self.module.critics, self.module.target_critics
+        update_polyak(critics, target_critics, self.config["polyak"])
         return info
 
     def _update_critic(self, batch_tensors):
