@@ -10,6 +10,8 @@ from ray.rllib.utils import override
 from torch.nn.utils import parameters_to_vector
 from torch.nn.utils import vector_to_parameters
 
+from raylab.options import configure
+from raylab.options import option
 from raylab.policy import TorchPolicy
 from raylab.policy.action_dist import WrapStochasticPolicy
 from raylab.torch.optim import build_optimizer
@@ -21,20 +23,48 @@ from raylab.utils.dictionaries import get_keys
 from raylab.utils.explained_variance import explained_variance
 
 
+LINESEARCH_DEFAULTS = {
+    "accept_ratio": 0.1,
+    "backtrack_ratio": 0.8,
+    "max_backtracks": 15,
+    "atol": 1e-7,
+}
+
+
+@configure
+@option("delta", 0.01, help="Trust region constraint")
+@option(
+    "fvp_samples",
+    10,
+    help="Number of actions to sample per state for Fisher vector-product calculation",
+)
+@option("lambda", 0.97, help=r"For GAE(\gamma, \lambda)")
+@option("val_iters", 80, help="Number of iterations to fit value function")
+@option("use_gae", True, help="Whether to use Generalized Advantage Estimation")
+@option("cg_iters", 10, help="Number of iterations for Conjugate Gradient")
+@option(
+    "cg_damping",
+    1e-3,
+    help="Damping factor to avoid singular matrix multiplication in Conjugate Gradient",
+)
+@option(
+    "line_search",
+    True,
+    help="""
+    Whether to use a line search to calculate policy update.
+    Effectively turns TRPO into Natural PG when turned off.
+    """,
+)
+@option("line_search_options", LINESEARCH_DEFAULTS)
+@option("module/type", "TRPO")
+@option("torch_optimizer", {"type": "Adam", "lr": 1e-3}, override=True)
+@option("exploration_config/type", "raylab.utils.exploration.StochasticActor")
 class TRPOTorchPolicy(TorchPolicy):
     """Policy class for Trust Region Policy Optimization."""
 
     # pylint:disable=abstract-method
     dist_class = WrapStochasticPolicy
 
-    @property
-    @override(TorchPolicy)
-    def options(self):
-        # pylint:disable=cyclic-import
-        from raylab.agents.trpo import TRPOTrainer
-
-        return TRPOTrainer.options
-
     @override(TorchPolicy)
     def _make_optimizers(self):
         optimizers = super()._make_optimizers()
