@@ -1,10 +1,14 @@
 """Mixins for model-based policies."""
 from .envfn import EnvFnMixin
+from .lightning import LightningModelMixin
+from .policy import MBPolicyMixin
 from .sampling import ModelSamplingMixin
 from .training import ModelTrainingMixin
 
 __all__ = [
-    "ModelTrainingMixin",
-    "ModelSamplingMixin",
     "EnvFnMixin",
+    "MBPolicyMixin",
+    "LightningModelMixin",
+    "ModelSamplingMixin",
+    "ModelTrainingMixin",
 ]
